//   Plagiat.java --- Fungus -- A multi agent simulation framework

//   Copyright (C) 2004 Moise Valvassori <djedi@ai.univ-paris8.fr>

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.

//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.djedi.plagiat;

import java.io.Reader;
import java.io.StreamTokenizer;
import java.io.IOException;
import org.djedi.snafu.Coordinator;
import org.djedi.snafu.Query;
import org.djedi.snafu.SearchResultEntry;
import org.djedi.snafu.ResultListener;
import org.djedi.snafu.ResultEvent;
import java.text.StringCharacterIterator;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;
import java.net.URL;
import sun.security.krb5.internal.crypto.e;
import java.util.TreeSet;

/**
 * Dtermine si un texte donn est un plagiat.
 * @author "Moise Valvassori" <djedi@ai.univ-paris8.fr>
 * @version "$Id: Plagiat.java,v 1.6 2004/05/15 09:27:41 djedi Exp $"
 * @since Thu Jan  8 16:16:58 2004
 */
public class Plagiat implements ResultListener{
  private String buffer;
  private ArrayList pl;
  private int ssentence = 5;
  private HashMap queries;

  /**
   * URL trouves par google --> tableaux ID segment
   */
  private HashMap urls;
  private HashMap urls_coef;
  /**
   * Coefs
   */
  private TreeSet coefs;
  private int maxresult = 10;


  /**
   * Creates a new <code>Plagiat</code> instance.
   *
   * @param s le buffer  analyser
   */
  public Plagiat (String s) {
    buffer = s;
    pl = new ArrayList();
    queries = new HashMap();
    urls = new HashMap();
  }


  public void addPlagiatListenner(PlagiatListenner l) {
    pl.add(l);
  }

  private void firePlagiatEvent(PlagiatEvent e) {
    for (Iterator i=pl.iterator(); i.hasNext();) {
      PlagiatListenner p = (PlagiatListenner)i.next();
      p.plagiatDetected(e);
    }
  }

  /**
   * Analyse un texte
   */
  public void doquerie () {
    StringCharacterIterator i= new StringCharacterIterator(buffer);
    boolean inWord = false;
    int startWord = 0;
    int nw = 0;
    Coordinator coord = new Coordinator();
    coord.addResultListenner(this);

    for (char c = i.first(); c != StringCharacterIterator.DONE; c = i.next()) {
      if (inWord){
	if (! Character.isLetterOrDigit(c)) {
	  inWord = false;
	  nw++;
	  if (nw == ssentence) {
	    queries.put(coord.search("\""+buffer.substring(startWord,i.getIndex())+"\"",maxresult),
			new Segment(startWord,i.getIndex()));
	    firePlagiatEvent(new PlagiatEvent(PlagiatEvent.PARSE,buffer,startWord,i.getIndex(),""));
	    //	    System.out.println(startWord+" "+i.getIndex()+" - "+buffer.substring(startWord,i.getIndex()));
	    nw=0;
	  }
	}}
      else
	if (Character.isLetterOrDigit(c)) {
	  inWord = true;
	  if (nw == 0)
	    startWord = i.getIndex();
	}
    }
  }

  /**
   * Analyse les resultats.
   * Pour toutes les URLs collectes, on valorise les segments proches:
   * @math $$\sum_i \frac{1}{s_i-s_{i-1}}$$
                               --    1   
                               \  -------
                               /  s -s   
                               --  i  i-1    
                               i 
   
   * 
   */
  public void analyse() {
    urls_coef = new HashMap();
    coefs = new TreeSet();
    for (Iterator it=urls.keySet().iterator(); it.hasNext();) {
      URL url = (URL)it.next();
      ArrayList l = (ArrayList)urls.get(url);
      float res = 1;
      for (int a=1; a<l.size(); a++) {
	float s0 = ((Integer)l.get(a-1)).floatValue();
	float s1 = ((Integer)l.get(a)).floatValue();
	res += 1/(s1-s0);
      }
      urls_coef.put(url,new Float(res));
      coefs.add(new Float(res));
      for (int a=0; a<l.size(); a++) {
	Integer id = ((Integer)l.get(a));
	Segment s = (Segment)queries.get(id);	  
	try {
	firePlagiatEvent(new PlagiatEvent(PlagiatEvent.COEF,buffer,s.getStart(),s.getEnd(),
					  Float.toString(res)+""+
					  url.toString()));
	} catch (NullPointerException e) {
	  System.err.println("******"+s+" "+id+" "+res+" "+e);
	  //	  e.printStackTrace();
	}
      }
    }
//     System.out.println(urls_coef);
//     System.out.println(coefs);
  }


  public void resultAdded(ResultEvent e) {
    System.out.println(e.getId()+" "+e.getEntry().getURL());
    Segment s = (Segment)queries.get(e.getId());

    ArrayList l = (ArrayList)urls.get(e.getEntry().getURL());
    if (l==null)
      l = new ArrayList();
    l.add(e.getId());
    urls.put(e.getEntry().getURL(),l);

    firePlagiatEvent(new PlagiatEvent(PlagiatEvent.FOUND,buffer,s.getStart(),s.getEnd(),""));
  }


  /**
   * Segments de texte
   * @author "Moise Valvassori" <djedi@ai.univ-paris8.fr>
   * @version "$Id: Plagiat.java,v 1.6 2004/05/15 09:27:41 djedi Exp $"
   * @since Fri Jan 16 15:45:11 2004
   */
  private class Segment {
    private int sta,end;
    public Segment (int s, int e) {
      sta = s;
      end = e;
    }
    public int getStart() {
      return sta;
    }
    public int getEnd() {
      return end;
    }
  }

  //   private Reader reader;
  //   private String [] buffer;
  //   private int size = 5;

  //   /**
  //    * Creates a new <code>Plagiat</code> instance.
  //    *
  //    */
  //   public Plagiat (Reader r) {
  //     reader = r;
  //     buffer = new String[size];
  //   }

  //   private String generate_keyword() {
  //     String s="";
  //     for (int i=0; i<size; i++) {
  //       s +=buffer[i];
  //       if (i != size-1)
  //	s+="+";
  //     }
  //     return s;
  //   }

  //   public void analyse() {
  //     StreamTokenizer tokenizer = new StreamTokenizer(reader);
  //     tokenizer.ordinaryChar('\'');
  //     int t=StreamTokenizer.TT_EOF;
  //     int i = 0;
  //     Coordinator coord = new Coordinator();
  //     while (true) {
  //       try {
  //	t = tokenizer.nextToken();
  //	//	System.out.println(t+" "+tokenizer);
  //	if (t == StreamTokenizer.TT_EOF)
  //	  break;
  //	if (t==StreamTokenizer.TT_WORD) {
  //	  buffer[i++] = tokenizer.sval;
  //	  //	  System.out.println(tokenizer.sval);
  //	  if (i==size) {
  //	    i=0;
  //	    System.out.println(generate_keyword());
  //	    Integer q = coord.search("\""+generate_keyword()+"\"",5);
  //	    System.out.println(q);
  //	    Query qq = coord.getQuery(q);
  //	    System.out.println(qq.size()+" -- "); //+(()qq.getResultList().get(0)));
  //	  }
  //	}
  //       } catch (IOException e) {
  //	e.printStackTrace();
  //       }
  //     }
  //     for (i=0; i<coord.getSize(); i++) {
  //       Query qq = coord.getQuery(new Integer(i));
  //       System.out.println(qq.size());
  //       if (qq.size() !=0)
  //	for (int j=0; j<qq.size(); j++)
  //	  System.out.println(((SearchResultEntry)qq.getResultList().get(j)));
  //     }
  //   }
}
