//   Coordinator.java --- Snafu

//   Copyright (C) 2002 Moise Valvassori <djedi@ai.univ-paris8.fr>

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.

//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.djedi.snafu;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.HashMap;
import java.util.LinkedList;

/**
 *
 * @author "Moise Valvassori" <djedi@ai.univ-paris8.fr>
 * @version 1.0
 * @since Thu Nov 21 16:14:00 2002
 */
public class Coordinator {
  private ArrayList engines;
  private HashMap queries;
  private LinkedList resultListenners;
  private int pageNumber = 25;


  /**
   * Creates a new <code>Coordinator</code> instance.
   *
   */
  public Coordinator () {
    engines = new ArrayList ();
    queries = new HashMap();
    resultListenners = new LinkedList();
    createEngines();
  }

  private void createEngines () {
    engines.add(new org.djedi.snafu.engines.GoogleEngine (this));
  }

  public Integer search(String word) {
    return search(word,pageNumber);
  }

  public Integer search(String word, int max) {
    Query q = new Query(word);
    queries.put(q.getId(),q);

    for (Iterator i = engines.iterator(); i.hasNext();) {
      Engine e = (Engine)i.next();
      e.search(word, max, q.getId());
    }
    return q.getId();
  }

  public Query getQuery(Integer qid) {
    return (Query)queries.get(qid);
  }

  public int getSize() {
    return queries.size();
  }

  public synchronized void collectResult(Integer qid, SearchResultEntry sr) {
    Query q = (Query)queries.get(qid);
    q.add(sr);
    fireResultEvent(new ResultEvent(this,qid,sr));
  }

  public void addResultListenner (ResultListener rl) {
    resultListenners.add(rl);
  }
  protected void fireResultEvent(ResultEvent e) {
    for (Iterator i = resultListenners.iterator(); i.hasNext();) {
      ResultListener l = (ResultListener)i.next();
      l.resultAdded(e);
    }
  }

  public void setNumberPages(int n) {
    pageNumber = n;
  }
}
