//   GoogleEngine.java --- Snafu

//   Copyright (C) 2002 Valvassori Mose <moise@valvassori.org>

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.

//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.djedi.snafu.engines;
import org.djedi.snafu.Engine;
import org.djedi.snafu.Coordinator;
import org.djedi.snafu.SearchResultEntry;
import java.net.URL;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.BufferedInputStream;
import java.net.HttpURLConnection;
import java.net.URLConnection;
import java.net.Socket;
import java.io.InputStream;
import java.util.LinkedList;
import java.net.URLEncoder;
import java.io.UnsupportedEncodingException;

/**
 *
 * @author "Valvassori Mose" <moise@valvassori.org>
 * @version $Id: GoogleEngine.java,v 1.8 2004/01/10 16:15:20 djedi Exp $
 * @since Thu Nov 21 10:28:13 2002
 */
public class GoogleEngine extends Thread implements Engine {
  private Coordinator coordinator;
  private LinkedList pool;
  /**
   * Creates a new <code>GoogleEngine</code> instance.
   *
   */
  public GoogleEngine (Coordinator c) {
    coordinator = c;
    pool = new LinkedList();
    start();
  }

  public synchronized void run () {
    while (true) {
      if (pool.isEmpty())
	try {
	  wait();
	} catch(Exception e) {
	  e.printStackTrace();
	  System.err.println(this);
	  System.exit(2);
	}
      else {
	PoolEntry p = (PoolEntry)pool.removeFirst();
	performSearch(p);
      }
    }
  }

  public synchronized void search(String word, Integer qid) {
    pool.add(new PoolEntry(word,qid,25));
    notify();
  }
  public synchronized void search(String word,int num, Integer qid) {
    pool.add(new PoolEntry(word,qid,num));
    notify();
  }

  private void performSearch(PoolEntry pe) {
    try {
      Socket s = new Socket ("www.google.com",80);
      //      System.out.println(pe.getNum());
      String query = URLEncoder.encode(pe.getWord(),"ISO-8859-1"); //"UTF-8");
      //      System.out.println("http://www.google.com/search?num="+pe.getNum()+"&q="+query);

      System.out.println("GET /search?num="+pe.getNum()+"&q="+query+"\n");
      s.getOutputStream().write(("GET /search?num="+pe.getNum()+"&q="+query+" HTTP/1.1\n\n").getBytes());

      GoogleParser p = new GoogleParser(new BufferedInputStream(s.getInputStream()));
      p.init(this, pe.getQid());
      int r;

      do  {
	r = p.yylex();
	
      }while (r != p.YYEOF);
      p = null;
    } catch (IOException e) {
      e.printStackTrace();
    }
  }

  public void newSearchResult(String url, String title, String desc, Integer qid) {
    coordinator.collectResult(qid,new SearchResultEntry(url,title,desc));
  }


  /**
   *
   * @author "Moise Valvassori" <djedi@ai.univ-paris8.fr>
   * @version 1.0
   * @since Wed Nov 27 16:26:39 2002
   */
  private class PoolEntry {
    private String word;
    private Integer qid;
    private int num;

    public PoolEntry (String word, Integer qid, int num) {
      this.word = word;
      this.qid = qid;
      this.num = num;
    }

    /**
     * Get the value of word.
     * @return value of word.
     */
    public String getWord() {
      return word;
    }

    /**
     * Get the value of qid.
     * @return value of qid.
     */
    public Integer getQid() {
      return qid;
    }
    public int getNum() {
      return num;
    }
  }
}
