//   QueryPane.java --- Snafu

//   Copyright (C) 2002 Valvassori Mose <moise@valvassori.org>

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.

//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.djedi.snafu.gui;

import java.awt.BorderLayout;
import java.awt.Component;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.TreeSet;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableModel;
import org.djedi.snafu.Coordinator;
import org.djedi.snafu.ResultEvent;
import org.djedi.snafu.ResultListener;
import org.djedi.snafu.SearchResultEntry;
import java.io.IOException;
import java.util.Collections;
import java.util.SortedSet;

/**
 *
 * @author "Valvassori Mose" <moise@valvassori.org>
 * @version $Id: QueryPane.java,v 1.4 2004/01/07 20:33:20 djedi Exp $
 * @since Sat Nov 23 22:38:55 2002
 */
public class QueryPane extends JPanel implements ResultListener {
  private Coordinator coordinator;
  private Integer qid;
  private JLabel l;
  private MyTableModel model;

  /**
   * Creates a new <code>QueryPane</code> instance.
   *
   */
  public QueryPane (Coordinator c, Integer qid) {
    coordinator = c;
    this.qid = qid;

    setLayout(new BorderLayout());
    JScrollPane sp = new JScrollPane();

    // Prepare la table
    Object [][]tmpmodel = {};
    model = new MyTableModel();
    JTable table = new JTable(model);
    table.setCellSelectionEnabled(false);
    table.setRowSelectionAllowed(true);
    table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
    ListSelectionModel lsm = table.getSelectionModel();
    lsm.addListSelectionListener( new ListSelectionListener() {
	public void valueChanged (ListSelectionEvent e) {
	  if (e.getValueIsAdjusting())
	    return ;
	  ListSelectionModel s = (ListSelectionModel)e.getSource();
	  try {
	    String cmd = "mozilla -remote \"openurl("+model.getValueAt(s.getMinSelectionIndex(),0)+")\"";
	    System.out.println(cmd);
	    Runtime.getRuntime().exec(cmd);
	    System.out.println("return");
	  } catch (IOException ex) {
	    ex.printStackTrace();
	  } // end of try-catch
	}
      });
    
    table.setDefaultRenderer(String.class,new MyCellRenderer());

    sp.setViewportView(table);
    this.add(sp,BorderLayout.CENTER);
  }

  public synchronized void resultAdded(ResultEvent e) {
    if (!qid.equals(e.getId()))
      return ;
    model.add(e.getEntry());
  }


  /**
   *
   * @author "Valvassori Mose" <moise@valvassori.org>
   * @version $Id: QueryPane.java,v 1.4 2004/01/07 20:33:20 djedi Exp $
   * @since Mon Nov 25 21:38:56 2002
   * @see TableModel
   */
  private class MyTableModel implements TableModel {
    private String [] colnames = {"URL","Title"}; 
    private SortedSet datas;

    public MyTableModel () {
      datas = Collections.synchronizedSortedSet(new TreeSet());
      listener = new ArrayList();
    }


    public synchronized void add(SearchResultEntry e) {
      datas.add(e);
      for (Iterator i = listener.iterator(); i.hasNext();) {
	((TableModelListener)i.next()).tableChanged(new TableModelEvent(this));
      }
    }

    public String getDesc(int row) {
      if ( row < datas.size()) {
	SearchResultEntry e = (SearchResultEntry)datas.toArray()[row];
	return e.getDesc();
      }
      return null;
    }

    public int getRowCount() {
      return datas.size();
    }
    public int getColumnCount() {
      return colnames.length;
    }
    public String getColumnName(int x) {
      return colnames[x];
    }
    public Class getColumnClass(int x) {
      return String.class;
    }
    public boolean isCellEditable(int r, int c) {
      return false;
    }
    public Object getValueAt(int r, int c) {
      if ( r < datas.size()) {
	SearchResultEntry e = (SearchResultEntry)datas.toArray()[r];
	if (c==0)
	  return e.getURL().toString();
	else if (c == 1)
	  return e.getTitle();
      }
      return null;
    }
    public void setValueAt(Object v, int r, int c) {
      
    }
    private ArrayList listener;
    public void addTableModelListener(TableModelListener l) {
      listener.add(l);
    }
    public void removeTableModelListener(TableModelListener l) {
      listener.remove(l);
    }
  }

  /**
   *
   * @author "Valvassori Mose" <moise@valvassori.org>
   * @version $Id: QueryPane.java,v 1.4 2004/01/07 20:33:20 djedi Exp $
   * @since Mon Nov 25 21:39:01 2002
   * @see DefaultTableCellRenderer
   * @see TableCellRenderer
   */
  private class MyCellRenderer extends DefaultTableCellRenderer
    implements TableCellRenderer {
    public Component getTableCellRendererComponent(JTable table, Object val, 
						   boolean sel, boolean focus, 
						   int row, int col) {
      MyTableModel m = (MyTableModel)table.getModel();
      this.setToolTipText((String)m.getDesc(row));
      Component c=  super.getTableCellRendererComponent(table,val,sel,focus,row,col);
      return c;
    }
  }
}
